<?php
    class BackupRestore {
        private static $tables = array(
            "BC_buchung",
            "BC_kategorie",
            "BC_konto",
            "BC_me_saldo",
            "BC_planposten",
            "BC_regel",
            "BC_textregel"
        );
        private static $columns = array(
            "BC_buchung" => ["id", "konto", "datum", "betrag", "text", "status", "kategorie", "ursprung"],
            "BC_kategorie" => ["id", "art", "bezeichnung", "sort", "parent"],
            "BC_konto" => ["id", "bezeichnung"],
            "BC_me_saldo" => ["id", "konto", "monat", "saldo"],
            "BC_planposten" => ["id", "konto", "beginn", "ende", "betrag", "text", "kategorie", "turnusArt", "turnusCount"],
            "BC_regel" => ["id", "sort", "keyString", "kategorie"],
            "BC_textregel" => ["id", "sort", "keyString", "toString"]
        );
        private static $colTypes = array(
            "BC_buchung" => ["num", "num", "char", "num", "char", "char", "num", "num"],
            "BC_kategorie" => ["num", "char", "char", "num", "num"],
            "BC_konto" => ["num", "char"],
            "BC_me_saldo" => ["num", "num", "num", "num"],
            "BC_planposten" => ["num", "num", "char", "char", "num", "char", "num", "char", "num"],
            "BC_regel" => ["num", "num", "char", "num"],
            "BC_textregel" => ["num", "num", "char", "char"]
        );

        private static function mkSqlSelect($table) {
            $res = "select ";
            $separ = "";
            foreach (self::$columns[$table] as $ix => $col) {
                $res .= $separ.$col;
                $separ = ", ";
            }
            return $res." from ".$table." where 1";
        }

        private static function mkSqlInsert($table, $line) {
            $values = explode("\t", $line);
            $res = "insert into ".$table;
            $separ = " ( ";
            foreach (self::$columns[$table] as $ix => $col) {
                $res .= $separ.$col;
                $separ = ", ";
            }
            $res .= " ) values";
            $separ = " ( ";
            foreach (self::$columns[$table] as $ix => $col) {
                $res .= $separ.$values[$ix];
                $separ = ", ";
            }
            return $res." )";
        }

        static function doBackup($file) {
            foreach (self::$tables as $ix => $table) {
                fwrite($file, ">>".$table."\n");
                $sql = new Sql(self::mkSqlSelect($table));
                $sql->execute();
                while ($row = $sql->fetch()) {
                    $line = "";
                    foreach (self::$columns[$table] as $ix => $col) {
                        if (self::$colTypes[$table][$ix] == "num")
                            $delim = "";
                        else
                            $delim = "'";
                        $line .= $delim.$row->$col.$delim."\t";
                    }
                    fwrite($file, $line."\n");
                }
            }
        }

        static function doRestore($file) {
            $table = "";
            while ($line = fgets($file)) {
                if (substr($line, 0, 2) == ">>") {
                    if ($table != "")
                        GlobalVars::push("messages", "Tabelle ".$table." - ".$count." Datensätze geladen.");
                    $table = trim(substr($line, 2));
                    $sql = new Sql("delete from ".$table." where 1");
                    $sql->execute();
                    $count = 0;
                } else {
                    if ($table == "")
                        BC_sayAndDie("Fehler: keine korrekte Dateistruktur in der Restore-Datei!");
                    if ($line != "") {
                        $sql = new Sql(self::mkSqlInsert($table, $line));
                        $sql->execute();
                        $count++;
                    }
                }
            }
            GlobalVars::push("messages", "Tabelle ".$table." - ".$count." Datensätze geladen.");
        }
    }
?>