<?php
class PlanungManager {

    private static function getFaelligDaten($pp, $von, $bis) {
        $res = array();
        if ($pp->getTurnusArt() == "e") {
            if ($pp->getBeginn()->isBetween($von, $bis))
                array_push($res, new Datum($pp->getBeginn()->getDispFormat()));
        } else {
            $dt1 = new Datum($pp->getBeginn()->getDispFormat());
            if ($dt1->isBefore($von)) {
                $diff = $pp->getBeginn()->diff($von);
                if ($pp->getTurnusArt() == "t") {
                    $diff = round(floor($diff / $pp->getTurnusCount()));
                    $dt1->addDays($diff * $pp->getTurnusCount());
                } else {
                    $diff = round(floor($diff / 31 / $pp->getTurnusCount()));
                    $dt1->addMonths($diff * $pp->getTurnusCount());
                }
            }
            $dt = new Datum($dt1->getDbFormat());
            $n = 1;
            while (!$dt->isAfter($bis) && !$dt->isAfter($pp->getEnde())) {
                if (!$dt->isBefore($von))
                    array_push($res, new Datum($dt->getDispFormat()));
                $dt->set($dt1->getDbFormat());
                if ($pp->getTurnusArt() == "t")
                    $dt->addDays($pp->getTurnusCount() * $n);
                else
                    $dt->addMonths($pp->getTurnusCount() * $n);
                $n++;
            }
        }
        return $res;
    }

    private static function makePlanBuchung($pp, $dt) {
        $bu = Buchung::neu();
        $bu->setKonto($pp->getKonto());
        $bu->getDatum()->set($dt->getDispFormat());
        $bu->setBetrag($pp->getBetrag());
        $bu->setText($pp->getText());
        $bu->setStatus("p");
        $bu->setKategorie($pp->getKategorie());
        $bu->setUrsprung($pp);
        $bu->save();
    }

    static function makePlanung($konto, $von, $bis) {
        Buchung::deletePlanBuchungen($konto);
        $ppListe = PlanPosten::getForKonto($konto);
        foreach ($ppListe as $ix => $pp) {
            $faelligDaten = self::getFaelligDaten($pp, $von, $bis);
            foreach ($faelligDaten as $ix1 => $dt)
                self::makePlanBuchung($pp, $dt);
        }
    }

    static function bereinigePlanung($konto, $stichtag) {
        $nEinmal = 0;
        $nPeriodisch = 0;
        $nAdjust = 0;
        $heute = new Datum("");
        $ppListe = PlanPosten::getForKonto($konto);
        foreach ($ppListe as $ix => $pp) {
            $ppBis = $pp->getEnde();
            if ($pp->getTurnusArt() == "e")
                $ppBis = $pp->getBeginn();
            if ($ppBis->isBefore($stichtag)) {
                if ($pp->getTurnusArt() == "e")
                    $nEinmal++;
                else
                    $nPeriodisch++;
                $pp->delete();
            } else if ($pp->getBeginn()->isBefore($stichtag)) {
                $dtListe = self::getFaelligDaten($pp, $stichtag, $heute);
                if (count($dtListe) > 0) {
                    $pp->setBeginn($dtListe[0]);
                    $pp->save();
                    $nAdjust++;
                }
            }
        }
        GlobalVars::push("messages", "Planung bereinigt für Konto ".$konto->getBezeichnung());
        GlobalVars::push("messages", "Anzahl gelöschte Einmal-Posten: ".$nEinmal);
        GlobalVars::push("messages", "Anzahl gelöschte Turnus-Posten: ".$nPeriodisch);
        GlobalVars::push("messages", "Anzahl justierte Turnus-Posten: ".$nAdjust);
    }
}
?>