<?php
    function getSqlInstallFiles() {
        $done = array();
        $todo = array();

        // Aus Update-Control-Datei lesen, welche Skripte schon verarbeitet wurden
        if (file_exists("config/updateCtl.txt")) {
            $file = fopen("config/updateCtl.txt", "r");
            while ($line = fgets($file)) {
                $parts = explode(" ", $line);
                $done[$parts[0]] = true;
            }
            fclose($file);
        }

        // In Script-Verzeichnis suchen, welche Skripte noch nicht verarbeitet wurden
        if (!$handle = opendir("./script"))
            die("Fehler beim Öffnen Verzeichnis script.");
        while (false !== ($entry = readdir($handle))) {
            if (substr($entry, -4) == ".sql" && !isset($done[$entry])) 
                array_push($todo, $entry);
        }
        closedir($handle);

        // Array noch zu verarbeitender Skripte zurückgeben
        sort($todo);
        return $todo;
    }

    function execSqlInstallFile($fname) {
        $file = fopen("./script/".$fname, "r");
        if (!$file)
            die("Fehler beim Öffnen Datei script/".$fnname);
        $stmt = "";
        $count = 0;
        while($line = fgets($file)) {
            if (substr($line, 0, 2) != "--")
                $stmt .= trim($line)." ";
            if (substr($stmt, -2) == "; ") {
                $sql = new Sql($stmt);
                $sql->execute();
                $stmt = "";
                $count++;
            }
        }
        fclose($file);
        $file = fopen("config/updateCtl.txt", "a");
        if (!$file)
            die("Fehler beim Öffnen Datei config/updateCtl.txt");
        fwrite($file, $fname." done\n");
        fclose($file);
        return "Update-Skript ".$fname." verarbeitet (".$count." Statements)";
    }

    function setPassword() {
        $pw = $_POST["pwneu1"];
        if ($pw != $_POST["pwneu2"])
            return "Fehler: Passwort-Eingaben sind nicht identisch!";
        if (strlen($pw) < 8)
            return "Fehler: Passwort ist zu kurz (mind. 8 Zeichen)!";
        $inFile = fopen("./util/LocalSettingsTemplate.php", "r");
        if (!$inFile)
            die("Fehler beim Öffnen Datei util/LocalSettingsTemplate.php");
        $outFile = fopen("./config/LocalSettings.php", "w");
        if (!$outFile)
            die("Fehler beim Öffnen Datei config/LocalSettings.php");
        while ($line = fgets($inFile)) {
            if (trim($line) == "###passhash###")
                $line = "    static \$passHash = \"".hash("sha256", $pw)."\";\n";
            fwrite($outFile, $line);
        }
        fclose($inFile);
        fclose($outFile);
        return "Passwort wurde gesetzt.";
    }

    function checkPassword($pw) {
        if (LocalSettings::$passHash != hash("sha256", $pw))
            return "Fehler: Falsches Passwort!";
        return "Passwort OK.";
    }

    function doInstallScripts() {
        $erg = array();
        require_once("./util/Sql.php");
        Sql::dbInit();
        $todo = getSqlInstallFiles();
        foreach ($todo as $ix => $instFile) {
            $message = execSqlInstallFile($instFile);
            array_push($erg, $message);
        }
        Sql::dbClose();
        return $erg;
    }

    // ********* Hier startet die Verarbeitungslogik der Seite *********
    $messages = array();
    $action = "none";
    if (isset($_POST["action"])) {
        // Seite wurde abgeschickt --> entsprechende Aktion durchführen
        $action = $_POST["action"];
        if ($action == "install") {
            $message = setPassword();
            array_push($messages, $message);
            if (substr($message, 0, 6) != "Fehler") {
                require_once("./config/LocalSettings.php");
                $messages = array_merge($messages, doInstallScripts());
            } else {
                $action = "none";
            }
        } else if ($action == "update") {
            require_once("./config/LocalSettings.php");
            $message = checkPassword($_POST["password"]);
            array_push($messages, $message);
            if (substr($message, 0, 6) != "Fehler")
                $messages = array_merge($messages, doInstallScripts());
            else
                $action = "none";
        } else if ($action == "setpw") {
            require_once("./config/LocalSettings.php");
            $message = checkPassword($_POST["password"]);
            array_push($messages, $message);
            if (substr($message, 0, 6) != "Fehler")
                array_push($messages, setPassword());
            else
                $action = "none";
        }
    }
    $modus = "install";
    $todo = getSqlInstallFiles();
    if (file_exists("./config/LocalSettings.php")) {
        $modus = "update";
        if (count($todo) == 0)
            $modus = "setpw";
    }
?>
<!doctype html>
<html lang="de">
<head>
    <meta charset="utf-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    <link rel="stylesheet" type="text/css" media="all" href="./style/style.css"/>
    <title></title>
</head>
<body>
	<form method="post">
		<table class="form" data-action="<?=$action?>">
            <tr>
				<th colspan="2" class="top">Installation / Update BControl</th>
			</tr>
<?php
    foreach ($messages as $ix => $message) { 
?>
            <tr>
				<td colspan="2" class="message"><?=$message?></td>
			</tr>
<?php 
    }
    if ($modus == "update" && $action == "none") {
?>
            <tr>
				<td>Aktuelles Passwort:</td>
				<td><input type="password" name="password" value=""></td>
			</tr>
<?php
    }
    if ($modus == "setpw" && $action == "none") {
?>
            <tr>
				<td colspan="2">
                    Wenn du dein Passwort vergessen hast, dann l&ouml;sche die
                    Datei <span class="mono">config/LocalSettings.php</span> und
                    lade diese Seite neu.<br>
                    Mit dem Zugriff auf das Dateisystem beweist du, dass du zum
                    Setzen eines neuen Passworts berechtigt bist.
                </td>
			</tr>
<?php 
    }
    if (count($todo) > 0) {
?>
			<tr>
                <td>Folgende Skripte stehen zur Ausf&uuml;hrung an:</td>
                <td>
<?php   foreach ($todo as $ix => $fname) { ?>
                    <span class"mono"><?=$fname?></span><br>
<?php   } ?>
                </td>
			</tr>
<?php
    }
    if ($modus == "install") {
?>
            <tr>
				<td colspan="2" class="bold">Zugangspasswort setzen (mind. 8 Zeichen lang)</td>
			</tr>
            <tr>
				<td>Neues Passwort:</td>
				<td><input type="password" name="pwneu1" value=""></td>
			</tr>
            <tr>
				<td>Neues Passwort wiederholen:</td>
				<td><input type="password" name="pwneu2" value=""></td>
			</tr>
<?php 
    }
    if ($action == "none") {
?>
			<tr>
				<td>&nbsp;</td>
				<td><button type="submit" name="action" value="<?=$modus?>">OK</button></td>
			</tr>
<?php 
    } else {
?>
			<tr>
				<td>Fertig</td>
				<td><a href="index.php">Hier</a>geht's los!</td>
			</tr>
<?php
    }
?>
		</table>
	</form>
</body>
</html>
