<?php

class Datum {
    private $date;
    private static $wochenTag = array(0 => "Sonntag", 1 => "Montag",
        2 => "Dienstag", 3 => "Mittwoch", 4 => "Donnerstag",
        5 => "Freitag", 6 => "Samstag");
    private static $monat = array(1 => "Januar", 2 => "Februar",
        3 => "M&auml;rz", 4 => "April", 5 => "Mai", 6 => "Juni",
        7 => "Juli", 8 => "August", 9 => "September", 10 => "Oktober",
        11 => "November", 12 => "Dezember");
  
    function __construct($datum) {
        $this->set($datum);
    }
    static function heute() {
        $obj = new Datum("");
        return $obj;
    }
    
    function set($datum) {
        if ($datum == "") {
            $this->date = new DateTime();
            return;
        };
        if ($datum == "unbekannt") {
            $this->date = new DateTime("1900-01-01");
            return;
        };
        $parts = explode(".", $datum);
        if (count($parts) == 1) {
            $parts = explode("-", $datum);
            $tag = $parts[2];
            $monat = $parts[1];
            $jahr = $parts[0];
        } else {
            $tag = $parts[0];
            $monat = $parts[1];
            $jahr = $parts[2];
        }
        if ($jahr < 100)
            $jahr += 2000;
        $this->date = new DateTime($jahr."-".$monat."-".$tag);
    }
    
    function getAsKey() {
        return $this->date->format("Ymd");
    }
    
    function getDbFormat() {
        return $this->date->format("Y-m-d");
    }
    function getDispFormat() {
        $erg = $this->date->format("d.m.Y");
        if ($erg == "01.01.1900")
            $erg = "unbekannt";
        return $erg;
    }
    
    function getWochentag() {
        return self::$wochenTag[$this->date->format("w")];
    }
    function getWoTag() {
        return substr(self::$wochenTag[$this->date->format("w")], 0, 2);
    }
    function getWoTagIndex() {
        $erg = $this->date->format("w");
        if ($erg == 0)
            return 7;
        return $erg;
    }
    
    function getTag() {
        return $this->date->format("d");
    }
    
    function getMonatNum() {
        return $this->date->format("m");
    }
  
    function getMonat() {
        $monat1 = $this->date->format("m") + 1;
        return self::$monat[$monat1 - 1];
    }
    
    function getDt() {
        return $this->date->format("d.m.");
    }
  
    function getAsMonat() {
        return $this->date->format("Y") * 100 + $this->date->format("m");
    }
  
    function getJahr() {
        return $this->date->format("Y");
    }
  
    function isBefore($d) {
        return $this->date->format("Ymd") < $d->date->format("Ymd");
    }
  
    function isAfter($d) {
        return $this->date->format("Ymd") > $d->date->format("Ymd");
    }
  
    function isSame($d) {
        return $this->date->format("Ymd") == $d->date->format("Ymd");
    }
    
    function isInFuture() {
        $d = new Datum("");
        return $d->isBefore($this);
    }

    function isBetween($d1, $d2) {
        if ($this->date->format("Ymd") < $d1->date->format("Ymd"))
            return false;
        if ($this->date->format("Ymd") > $d2->date->format("Ymd"))
            return false;
        return true;
    }
  
    function addDays($n) {
        $this->date->modify($n." days");
    }
  
    function addMonths($n) {
        $day = $this->date->format("d");
        $this->date->modify($n." month");
        if ($this->date->format("d") != $day) {
            // Workaround für PHP-Bug: Fehler wenn Ziel-Monat weniger Tage hat als Quell-Monat
            $month = $this->date->format("m");
            while ($this->date->format("m") == $month)
                $this->date->modify("-1 days");
        }
    }
  
    function setToMonatsErster() {
        $jahr = $this->date->format("Y");
        $monat = $this->date->format("m");
        $this->date->setDate($jahr, $monat, 1);
    }
  
    function setToMonatsLetzter() {
        $this->setToMonatsErster();
        $this->date->modify("+1 month");
        $this->date->modify("-1 days");
    }
  
    function setFromMonat($monat) {
        $m = $monat % 100;
        $y = floor($monat / 100);
        $this->date = new DateTime($y."-".$m."-1");
    }
    
    function diff($d) {
        $diff = $this->date->diff($d->date);
        return $diff->days;
    }

} // Ende Klasse Datum
?>