<?php
class ME_Saldo {
    // **** Konten-Saldo am Monatsende ****
    private static $buffer = array();
    private $persistent;
    private $id;
    private $konto;
    private $kontoId;
    private $monat;
    private $saldo;

    // **************** generierte statische Methoden ***************

    // Neues Objekt anlegen mit ID-Vergabe:
    static function neu() {
        $obj = new ME_Saldo();
        $sql = new Sql("select max(id) from BC_me_saldo");
        $obj->id = $sql->selectSingleValue();
        if ($obj->id == "")
            $obj->id = 0;
        $obj->id++;
        return $obj;
    }

    // Suche mit ID, Liefere Objekt oder NULL:
    static function findById($id) {
        if (isset(self::$buffer[$id]))
            return self::$buffer[$id];
        $obj = NULL;
        $sql = new Sql("select id, konto, monat, saldo from BC_me_saldo");
        $sql->append(" where id = ".$id);
        $sql->execute();
        if ($row = $sql->fetch()) {
            $obj = new ME_Saldo();
            $obj->setFromRow($row);
            self::$buffer[$obj->id] = $obj;
            $sql->free();
        }
        return $obj;
    }

    // Suche mit Where-Klausel und liefere Array von Objekten:
    private static function findByClause($clause) {
        $erg = array();
        $sql = new Sql("select id, konto, monat, saldo from BC_me_saldo ".$clause);
        $sql->execute();
        while ($row = $sql->fetch()) {
            if (isset(self::$buffer[$row->id]))
                array_push($erg, self::$buffer[$row->id]);
            else {
                $obj = new ME_Saldo();
                $obj->setFromRow($row);
                array_push($erg, $obj);
                self::$buffer[$obj->id] = $obj;
            }
        }
        return $erg;
    }

    // **************** generierte Objekt-Methoden ***************

    function __construct () {
        $this->persistent = false;
        $this->id = 0;
        $this->konto = null;
        $this->kontoId = 0;
        $this->monat = new Monat(0);
        $this->saldo = 0;
    }

    function equals($obj) {
        return (get_class() == get_class($obj) &&
                $this->id == $obj->getId());
    }

    // Attribute aus Query-Ergebniszeile setzen:
    private function setFromRow($row) {
        $this->persistent = true;
        $this->id = $row->id;
        $this->kontoId = $row->konto;
        $this->konto = null;
        $this->monat->set($row->monat);
        $this->saldo = $row->saldo;
    }

    // Speichern der Objekt-Attribute per Update oder Insert:
    function save() {
        if ($this->persistent) {
            $sql = new Sql("update BC_me_saldo");
            $sql->append(" set konto = ".$this->kontoId);
            $sql->append(" , monat = ".$this->monat->getAsNumber());
            $sql->append(" , saldo = ".$this->saldo);
            $sql->append(" where id = ".$this->id);
        } else {
            $sql = new Sql("insert into BC_me_saldo");
            $sql->append(" ( id");
            $sql->append(" , konto");
            $sql->append(" , monat");
            $sql->append(" , saldo");
            $sql->append(" ) ");
            $sql->append("values");
            $sql->append(" ( ".$this->id);
            $sql->append(" , ".$this->kontoId);
            $sql->append(" , ".$this->monat->getAsNumber());
            $sql->append(" , ".$this->saldo);
            $sql->append(" )");
        }
        $sql->execute();
        $this->persistent = true;
        self::$buffer[$this->id] = $this; // Buffer aktualisieren
    }

    // Objekt von Datenbank loeschen:
    function delete() {
        if (!$this->persistent)
            return false;
        $sql = new Sql("delete from BC_me_saldo");
        $sql->append(" where id = ".$this->id);
        $sql->execute();
        $this->persistent = false;
    }

    function getId() {
        return $this->id;
    }
    function getKonto() {
        if ($this->kontoId == 0)
            return null;
        if (is_null($this->konto))
            $this->konto = Konto::findById($this->kontoId);
        return $this->konto;
    }
    function setKonto($param) {
        $this->konto = $param;
        if (is_null($this->konto))
            $this->kontoId = 0;
        else
            $this->kontoId = $this->konto->getId();
    }
    function getMonat() {
        return $this->monat;
    }
    function setMonat($param) {
        $this->monat = $param;
    }
    function getSaldo() {
        return $this->saldo;
    }
    function setSaldo($param) {
        $this->saldo = $param;
    }

    // **************** Ende generierte Methoden ***************

    static function getForMonat($konto, $monat) {
        $clause = " where konto = ".$konto->getId()." and monat = ".$monat->getAsNumber();
        $res = self::findByClause($clause);
        if (count($res) > 0)
            return reset($res);
        
        // Für gewünschten Monat kein Satz vorhanden --> neues Objekt erstellen
        // und ggf. Saldo aus Vormonat ermitteln
        $obj = self::neu();
        $obj->setKonto($konto);
        $obj->monat = $monat;
        $obj->save();
        $vObj = $obj->getVormonat();
        if (is_null($vObj))
            return $obj;      // vorher nichts da --> Satz mit Saldo 0 passt

        // Vor-Saldo vorhanden --> neuen Saldo aus Buchungen errechnen
        $obj->aktualisiereAusVormonat($vObj);
        return $obj;
    }

    private function getVormonat() {
        $clause = " where konto = ".$this->konto->getId()." and monat < ".$this->monat->getAsNumber().
                " order by monat desc";
        $res = self::findByClause($clause);
        if (count($res) == 0)
            return null;
        else
            return reset($res);
    }

    private function aktualisiereAusVormonat($vormonat) {
        $von = $vormonat->getMonat()->getMonatsLetzter();
        $von->addDays(1);     // Start-Datum für relevante Buchungen
        $bis = $this->monat->getMonatsLetzter();    // End-Datum für relevante Buchungen
        $bListe = Buchung::findByParms($this->konto, "a", $von, $bis);
        $this->saldo = $vormonat->saldo;
        foreach ($bListe as $ix => $b)
            $this->saldo += $b->getBetrag();
        $this->save();
    }

    function aktualisiere() {
        $vObj = $this->getVormonat();
        if (is_null($vObj)) {
            $vObj = new ME_Saldo();
            $vObj->monat->set(190001);
        }
        $this->aktualisiereAusVormonat($vObj);
    }

} // Ende Klasse ME_Saldo
?>
