<?php

    class Sql {
        private static $conn = null;
        private $stmt;
        private $res;
    
        // Dabenbank-Verbindung aufbauen und Transaktion starten:
        static function dbInit() {
            $ver = (float)phpversion();
            self::$conn = new mysqli(LocalSettings::$dbServer, LocalSettings::$dbUser
                                   , LocalSettings::$dbPassword, LocalSettings::$dbDatabase);
            if (self::$conn->connect_error) {
                die("Datenbank-Fehler (".self::$conn->connect_errno.") ".self::$conn->connect_error."<br>\n");
            }
            self::$conn->autocommit(false);
            if ($ver >= 7.0)
                self::$conn->begin_transaction();
            self::$conn->set_charset("utf8");
            return;
        }

        // Konstruktor für neues SQL-Statement:
        function __construct($txt) {
            $this->stmt = $txt;
            $this->res = NULL;
        }

        // Fehlerabhandlung nach DB-Zugriffen:
        private function handleError() {
            if (self::$conn->error) {
                if (self::$conn->errno == 1050)
                    return false; // Table already exists
                echo "<br>Datenbank-Zugriffsfehler bei ".$this->stmt."<br>\n";
                echo "--&gt; Fehler (".self::$conn->errno.") ".self::$conn->error."<br>\n";
                self::rollback();
                debug_print_backtrace();
                die("<h1>Transaction Rollback!</h1>");
            }
            return true;
        }

        // Weiteren Teil an SQL-Statement anfügen:
        function append($txt) {
            $this->stmt = $this->stmt.$txt;
        }

        // SQL-Statement ausführen mit Fehlerabhandlung und ggf. Logging.
        // Ergebnismenge kann mit sql.fetch() abgearbeitet werden.
        function execute() {
            if ($this->stmt == "") {
                echo "<br>Warnung: leeres SQL-Statement nicht ausgef&uuml;hrt!<br>\n";
                return false;
            }
            if (LocalSettings::$traceSql)
                $start = microtime(true);
            $this->res = self::$conn->query($this->stmt);
            if (LocalSettings::$traceSql) {
                $end = microtime(true);
                $logFile = fopen(GlobalVars::getVar("PathPrefix")."data/sqlTrace".$userId.".txt", "a");
                $elapsed = number_format($end - $start, 6, ",", ".");
                fputs($logFile, date("Y-m-d h:i:s")."\t".$elapsed."\t".$this->stmt."\n");
                fclose($logFile);
            }
            return $this->handleError();
        }
    
        // Naechsten Satz aus Ergebnismenge liefern:
        function fetch() {
            if ($row = $this->res->fetch_object())
                return $row;
            $this->res->close();
            return false;
        }
    
        // Free auf Resultset machen, falls nicht fetch() bis EOD.
        function free() {
            // echo "sql->free() für ".$this->stmt."<br>\n";
            $this->res->close();
        }
    
        // SQL-Befehl ausfuehren, der einen einzelnen Wert liefert.
        // SQL-Ergebnis als Funktionsergebnis liefern.
        function selectSingleValue() {
            $this->execute();
            $this->handleError();
            if ($row = $this->res->fetch_array()) {
                foreach($row as $i => $val) {
                    $erg = $val;
                }
            } else
                $erg = "";
            $this->res->close();
            return $erg;
        }
    
        // Datenbank-Aenderungen ruecksetzen:
        static function rollback() {
            self::$conn->rollback();
            self::$conn->close();
        }
    
        // Transaktion commiten und Datenbankverbindung schliessen:
        static function dbClose() {
            self::$conn->commit();
            self::$conn->close();
        }

    } // Ende Klasse Sql
?>